@@ -26,6 +26,14 @@ class Workflow(CacheableBase):
         self.age = self.getAge()
         self.errors = self.getErrors( getUnreported=getUnreportedErrors )
         
+    def serialize(self):
+        """
+        the output of this method is used by the rest-api in the "AdditionalInfo" field
+        """
+        return { 
+            'PrepID':self.getPrepID()
+        }
+
     def __str__(self):
         return 'workflowinfo_%s' % self.workflowName
         
@@ -116,13 +124,49 @@ class Workflow(CacheableBase):
         :returns: Age of the workflow
         :rtype: float
         """
+        if 'RequestTransition' not in self.workflowParams:
+            return "Age unknown"
         for transition in self.workflowParams['RequestTransition']:
             if transition['Status'] == 'assignment-approved':
                 return int(time.time()) - int(transition['UpdateTime'])
 
         raise Exception("Failed to get the age of workflow: The workflow has no assignment-approved history")
 
+    def getJobStats(self):
+        """
+        :param None
+        :returns: a dictionary containing number of successful and failed jobs for each task/step in the following format::
+                  {<taskName>: 
+                    nSuccess: X,
+                    nFailure: Y
+                  }
+        :rtype: dict
+        """
 
+        response = getResponse(url=self.url,
+                             endpoint='/wmstatsserver/data/request/',
+                             param=self.workflowName)
+        
+        jobStatsPerTask = {}
+        if not response['result']:
+            return jobStatsPerTask
+
+        for agentName, agentData in response['result'][0].get(self.workflowName)['AgentJobInfo'].items():
+            for taskName, taskData in agentData['tasks'].items():
+                # Some tasks such as LogCollect, Cleanup etc. don't have job info
+                # TODO: Decide whether we should ignore such tasks or not. Ignore for now
+                if 'status' not in  taskData:
+                    continue
+                else:
+                    taskStatus = taskData['status']
+                nSuccess = taskStatus['success'] if 'success' in taskStatus else 0
+                nFailure = sum(taskStatus['failure'].values()) if 'failure' in taskStatus else 0
+                if taskName in jobStatsPerTask:
+                    jobStatsPerTask[taskName]['nSuccess'] += nSuccess
+                    jobStatsPerTask[taskName]['nFailure'] += nFailure
+                else:
+                    jobStatsPerTask[taskName] = {'nSuccess': nSuccess, 'nFailure': nFailure} 
+        return jobStatsPerTask
 
     def getFailureRate(self):
         """
@@ -131,7 +175,16 @@ class Workflow(CacheableBase):
                   {task: failure_rate}
         :rtype: dict
         """
-        pass
+        failureRatePerTask = {}
+        jobStats = self.getJobStats()
+        for taskName, stats in jobStats.items():
+            # Avoid division by zero, although we shouldn't have such data
+            if stats['nFailure'] == 0 and stats['nSuccess'] == 0:
+                failureRatePerTask[taskName] = -1 
+            else:
+                failureRatePerTask[taskName] = stats['nFailure'] / (stats['nFailure'] + stats['nSuccess'])
+        return failureRatePerTask
+   
 
     ## Get request related values
 
